//------------------------------------------------------------------------------
// <copyright>
//   Copyright (c) 2006 Nesterovsky bros. All rights reserved.
//   Home: http://www.nesterovsky-bros.com
//   E-mail: mailto:contact@nesterovsky-bros.com
// </copyright>
// <description>
//   Custom functoids for BizTalk Server 2006.
// </description>
// <history>
//  Created: 15 Jun 2006, Arthur & Vladimir Nesterovsky
// </history>
//------------------------------------------------------------------------------

namespace NesterovskyBros.BizTalk.Functoids
{
	using System;
	using System.Reflection;
	using System.Text;
	using System.Collections;
	using System.Globalization;
	using System.Text.RegularExpressions;
	using Microsoft.BizTalk.BaseFunctoids;

	/// <summary>
	/// ReplaceFunctoid allows using simple regular expressions for
	/// transformation input string value. It takes exactly four inputs.
	/// </summary>
	public class ReplaceFunctoid: BaseFunctoid
	{
		public ReplaceFunctoid()
		{
			//ID for this functoid
			ID = 0x415601;

			// resource assembly must be ProjectName.ResourceName if building 
			// with VS.Net
			SetupResourceAssembly("NesterovskyBros.BizTalk.Functoids.Resources", 
				Assembly.GetExecutingAssembly());

			// setup the Name, ToolTip, Help Description, and the Bitmap for 
			// this functoid
			SetName("ReplaceFunctoid_Name");
			SetDescription("ReplaceFunctoid_Description");
			SetTooltip("ReplaceFunctoid_ToolTip");
			SetBitmap("ReplaceFunctoid_Bitmap");
                 
			// set min and max parameters counters
			SetMinParams(3);
			SetMaxParams(3);

			// set the function name that needs to be called when this Functoid
			// is invoked. This means that this Functoid assembly need to be present 
			// in GAC for its availability during Test, Map and Runtime.
			SetExternalFunctionName(GetType().Assembly.FullName,
				"NesterovskyBros.BizTalk.Functoids.ReplaceFunctoid", "Apply");

			// specify a category for this functoid. This functoid goes under the 
			// String Functoid Tab in the VS.Net toolbox for functoids.
			Category = FunctoidCategory.String;

			// specify input and output parameters types
			AddInputConnectionType(ConnectionType.AllExceptRecord);
			AddInputConnectionType(ConnectionType.AllExceptRecord);
			AddInputConnectionType(ConnectionType.AllExceptRecord);
			AddInputConnectionType(ConnectionType.AllExceptRecord);

			OutputConnectionType = ConnectionType.AllExceptRecord;
		}

		/// <summary>
		/// Implements the ReplaceFunctoid logic.
		/// </summary>
		/// <param name="source">A source string.</param>
    /// <param name="pattern">A search string or regular expression pattern.</param>
    /// <param name="replace">Replacement text or regular expression pattern.</param>
		/// <returns>A string with substitutions.</returns>
		public string Apply(string source, string pattern, string replace)
		{
      if (string.IsNullOrEmpty(source))
        return null;

      if (string.IsNullOrEmpty(pattern))
        goto BadPattern;

      if (pattern[0] != '/')
        return source.Replace(pattern, replace);

      Regex regex = null;
      RegexOptions regexOptions = RegexOptions.CultureInvariant;
      bool global = false;
      int i = pattern.Length;

      while(i-- > 0)
      {
        switch(pattern[i])
        {
          case '/':
            // Count \ slashes.
            bool even = true;

            for(int j = i; j-- > 0;)
            {
              if (pattern[j] == '\\')
                even = !even;
              else
                break;
            }

            if (!even)
              goto BadPattern;

			      regex = new Regex(pattern.Substring(1, i - 1), regexOptions);

            goto Match;
          case 'g':
            global = true;

            break;
          case 'i':
            regexOptions |= RegexOptions.IgnoreCase;

            break;
          case 'm':
            regexOptions |= RegexOptions.Multiline;
                
            break;
          default:
            goto BadPattern;
        }
      }

    BadPattern:
      throw new ArgumentException("pattern");

    Match:
      MatchEvaluator evaluator = delegate(Match match) 
      { 
        return match.Result(replace); 
      };

      return global ? 
        regex.Replace(source, evaluator) :
        regex.Replace(source, evaluator, 1);
		}
	}
}